import { describe, it, expect, beforeEach } from 'vitest';
import {CelParser} from "../cel_parser";

// Tests
describe('Basics', async () => {
    let parser: any;
    beforeEach(async () => {
        parser = new CelParser();
    })

    it('Should return 0 when expression 0', async () => {
        const expression = "0";
        const expected = {int64_value: 0};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return 0 when expression 0 with spaces around', async () => {
        const expression = "  0   ";
        const expected = {int64_value: 0};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return 0 when expression 0u', async () => {
        const expression = "0u";
        const expected = {uint64_value: 0};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return 0 when expression 0U', async () => {
        const expression = "0U";
        const expected = {uint64_value: 0};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return 0 when expression 0.0', async () => {
        const expression = "0.0";
        const expected = {double_value: 0};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return empty string when expression r""', async () => {
        const expression = 'r""';
        const expected = {string_value: ""};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return empty string when expression b""', async () => {
        const expression = 'b""';
        const expected = {bytes_value: ""};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return banana string when expression r"banana"', async () => {
        const expression = 'r"banana"';
        const expected = {string_value: "banana"};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it("Should return banana string when expression r'banana'", async () => {
        const expression = "r'banana'";
        const expected = {string_value: "banana"};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return empty bytes value when expression b""', async () => {
        const expression = 'b""';
        const expected = {bytes_value: ""};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return true when expression true', async () => {
        const expression = "true";
        const expected = {bool_value: true};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return false when expression false', async () => {
        const expression = "false";
        const expected = {bool_value: false};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return null when expression null', async () => {
        const expression = "null";
        const expected = {null_value: null};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return list with values [] when expression is []', async () => {
        const expression = '[]';
        const expected = {
            list_value: {}
        };
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return list with values [] when expression is [] with spaces', async () => {
        const expression = ' [] ';
        const expected = {
            list_value: {}
        };
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return map_value with values {} when expression is {}', async () => {
        const expression = '{}';
        const expected = {
            map_value: {}
        };
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return 42 when expression 42', async () => {
        const expression = "42";
        const expected = {int64_value: 42};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return 123456789 when expression 123456789u', async () => {
        const expression = "123456789u";
        const expected = {uint64_value: 123456789};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return 123456789 when expression 123456789U', async () => {
        const expression = "123456789U";
        const expected = {uint64_value: 123456789};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return -9223372036854775808 when expression -9223372036854775808', async () => {
        const expression = "-9223372036854775808";
        const expected = {int64_value: -9223372036854775808};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return -23.0 when expression -2.3e+1', async () => {
        const expression = "-2.3e+1";
        const expected = {double_value: -23.0};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return ! when expression "!"', async () => {
        const expression = "\"!\"";
        const expected = {string_value: "!"};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return \' when expression "\'\\\'""', async () => {
        const expression = "'\\''";
        const expected = {string_value: "'"};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it("Should return \\255\\97 when expression b'ÿa'", async () => {
        const expression = "b'ÿa'";
        const expected = {bytes_value: "\\255\\97"};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return \\255\\97 when expression b"ÿa"', async () => {
        const expression = 'b"ÿa"';
        const expected = {bytes_value: "\\255\\97"};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return list with values with single item when expression is [-1]', async () => {
        const expression = '[-1]';
        const expected = {
            list_value: {
                values: [
                    {
                        int64_value: -1
                    }
                ]
            }
        };
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return map_value with matched values when expression is {"k": "v"}', async () => {
        const expression = '{"k": "v"}';
        const expected = {
            "map_value": {
                "entries": [
                    {
                        "key": {
                            "string_value": "k"
                        },
                        "value": {
                            "string_value": "v"
                        }
                    }
                ]
            }
        };
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return 1431655765 when expression 0x55555555', async () => {
        const expression = "0x55555555";
        const expected = {int64_value: 1431655765};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return -1431655765 when expression -0x55555555', async () => {
        const expression = "-0x55555555";
        const expected = {int64_value: -1431655765};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return 1431655765 when expression 0x55555555u', async () => {
        const expression = "0x55555555u";
        const expected = {uint64_value: 1431655765};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return -1431655765 when expression -0x55555555u', async () => {
        const expression = "-0x55555555u";
        const expected = {uint64_value: -1431655765};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return 1431655765 when expression 0x55555555U', async () => {
        const expression = "0x55555555U";
        const expected = {uint64_value: 1431655765};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return -1431655765 when expression -0x55555555U', async () => {
        const expression = "-0x55555555U";
        const expected = {uint64_value: -1431655765};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return 200 when expression 2e+2', async () => {
        const expression = "2e+2";
        const expected = {int64_value: 200};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return -1300 when expression -1.3e+2', async () => {
        const expression = "-13e+2";
        const expected = {int64_value: -1300};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return banana when expression banana', async () => {
        const expression = '"banana"';
        const expected = {string_value: "banana"};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return testing testing when expression testing testing', async () => {
        const expression = '"testing testing"';
        const expected = {string_value: "testing testing"};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return aa + 12 when expression aa + 12', async () => {
        const expression = '"aa + 12"';
        const expected = {string_value: "aa + 12"};
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return list with values string when expression is [1, "1", 8]', async () => {
        const expression = '[1, "1", 8]';
        const expected = {
            list_value: {
                values: [
                    {
                        int64_value: 1
                    },
                    {
                        string_value: "1"
                    },
                    {
                        int64_value: 8
                    }
                ]
            }
        };
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    it('Should return map_value with matched values when expression is {"bla": 1, "draa": 1, "test": 4, "another": "bla"}', async () => {
        const expression = '{"bla": 1, "draa": 1, "test": 4, "another": "bla"}';
        const expected = {
            "map_value": {
                "entries": [
                    {
                        "key": {
                            "string_value": "bla"
                        },
                        "value": {
                            "int64_value": 1
                        }
                    },
                    {
                        "key": {
                            "string_value": "draa"
                        },
                        "value": {
                            "int64_value": 1
                        }
                    },
                    {
                        "key": {
                            "string_value": "test"
                        },
                        "value": {
                            "int64_value": 4
                        }
                    },
                    {
                        "key": {
                            "string_value": "another"
                        },
                        "value": {
                            "string_value": "bla"
                        }
                    }
                ]
            }
        };
        expect(parser.parseString(expression)).toStrictEqual(expected);
    });

    //Errors
    it('Should throw a error when -1.3e+1.5 contains a dot', async () => {
        const expression = "-1.3e+1.5";
        const expectedFound = ".";
        const expectedLocation = 8;
        try {
            parser.parseString(expression)
        } catch(e: any) {
            expect(e.found).toBe(expectedFound)
            expect(e.location.start.column).toBe(expectedLocation)
        }
    });

    it('Should throw a error when 1.3e+1.5 as a double contains a e', async () => {
        const expression = "1.3e+1.5";
        const expectedFound = ".";
        const expectedLocation = 7;
        try {
            parser.parseString(expression)
        } catch(e: any) {
            expect(e.found).toBe(expectedFound)
            expect(e.location.start.column).toBe(expectedLocation)
        }
    });
});
