import {beforeEach, describe, expect, it} from "vitest";
import {CelParser} from "../cel_parser";

describe('Custom Types', async () => {
    let parser: any;

    const values = `{
        "data_points": {
            "banana": {"score": 1.5 },
            "banana_second": {"score": 2.0 },
            "real_name": {"score": 2},
            "real_name_second": {"score": 3},
            "array": [3, 5]
        },
        "data_Points": {
            "bAnana": {"score": 2.5 },
            "bing": {"score": "2.5" },
            "bong": {"score": true }
        },
        "formulas": {
            "formula_one": {"score": 3.5},
            "formula_two": {"score": 3},
            "forMula_three": {"score": 5}
        }
    }`;

    beforeEach(async () => {
        parser = new CelParser();
    })

    describe('Single custom type', async () => {
        it('Should return 1.5 when expression data_points.banana.score', async () => {
            const expression = "data_points.banana.score";
            const expected = {double_value: 1.5};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return string 2.5 when expression data_Points.bing.score', async () => {
            const expression = "data_Points.bing.score";
            const expected = {string_value: "2.5"};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return true when expression data_Points.bong.score', async () => {
            const expression = "data_Points.bong.score";
            const expected = {bool_value: true};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return 2 int64_value when expression data_points.real_name.score', async () => {
            const expression = "data_points.real_name.score";
            const expected = {int64_value: 2};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return 3.5 when expression formulas.formula_one.score', async () => {
            const expression = "formulas.formula_one.score";
            const expected = {double_value: 3.5};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return 2.0 when expression data_points.banana_second.score', async () => {
            const expression = "data_points.banana_second.score";
            const expected = {double_value: 2};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return 3 int64_value when expression formulas.formula_two.score', async () => {
            const expression = "formulas.formula_two.score";
            const expected = {int64_value: 3};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return 5 when expression formulas.forMula_three.score', async () => {
            const expression = "formulas.forMula_three.score";
            const expected = {int64_value: 5};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });
    })

    describe('Compare custom type fields', async () => {
        it('Should return false when expression data_points.banana.score > data_points.real_name.score', async () => {
            const expression = "data_points.banana.score > data_points.real_name.score";
            const expected = {bool_value: false};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return false when expression data_points.banana.score + formulas.formula_one.score > data_points.real_name.score', async () => {
            const expression = "data_points.banana.score + formulas.formula_one.score > data_points.real_name.score";
            const expected = {bool_value: true};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });
    })

    describe('Calculate custom type fields', async () => {
        it('Should return 3.5 when expression data_points.banana.score + data_points.banana_second.score', async () => {
            const expression = "data_points.banana.score + data_points.banana_second.score";
            const expected = {double_value: 3.5};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return -0.5 when expression data_points.real_name.score - data_points.real_name_second.score', async () => {
            const expression = "data_points.real_name.score - data_points.real_name_second.score";
            const expected = {int64_value: -1};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });
    })

    describe('Calculate custom type fields with Errors', async () => {
        it('when expression data_points.banana.score + data_points.real_name.score', async () => {
            const expression = "data_points.banana.score + data_points.real_name.score";
            const expectedFound = "You are trying a mathematical expression with mismatching types";
            try {
                parser.parseStringWithOptions(expression, values);
            } catch(e: any) {
                expect(e.message).toBe(expectedFound);
            }
        });

        it('when expression data_points.banana.score - data_points.real_name.score', async () => {
            const expression = "data_points.banana.score - data_points.real_name.score";
            const expectedFound = "You are trying a mathematical expression with mismatching types";
            try {
                parser.parseStringWithOptions(expression, values);
            } catch(e: any) {
                expect(e.message).toBe(expectedFound);
            }
        });
    })

    describe('Conditional custom fields', async () => {
        it('Should return 3.5 when expression data_points.banana.score + data_points.real_name.score < 4 ? data_points.banana.score + data_points.real_name.score : 4', async () => {
            const expression = "data_points.banana.score + data_points.banana_second.score < 4 ? data_points.banana.score + data_points.banana_second.score : 4";
            const expected = {double_value: 3.5};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return 2 when expression data_points.banana.score + data_points.real_name.score < 2 ? data_points.banana.score + data_points.real_name.score : 2', async () => {
            const expression = "data_points.real_name_second.score + data_points.real_name.score < 2 ? data_points.real_name_second.score + data_points.real_name.score : 2";
            const expected = {int64_value: 2};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });
    })

    describe('Conditional in check', async () => {
        it('Should return true when expression 5 in data_points.array', async () => {
            const expression = "5 in data_points.array";
            const expected = {bool_value: true};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return false when expression 5 in data_points.array', async () => {
            const expression = "2 in data_points.array";
            const expected = {bool_value: false};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });
    })
})
