import {beforeEach, describe, expect, it} from "vitest";
import {CelParser} from "../cel_parser";

describe('Logical', async () => {
    let parser: any;
    beforeEach(async () => {
        parser = new CelParser();
    })

    describe('And.true statements', async () => {
        it('Should return true when expression true && true', async () => {
            const expression = "true && true";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression true && true && true', async () => {
            const expression = "true && true && true";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 5 in [1, 4, 5] && 2 > 1', async () => {
            const expression = "5 in [1, 4, 5] && 2 > 1";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 5 in [1, 4, 5] && 2 > 1 && 3 == 3 && true && 2 <= 2', async () => {
            const expression = "5 in [1, 4, 5] && 2 > 1 && 3 == 3 && true && 2 <= 2";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('And.false statements', async () => {
        it('Should return false when expression true && false', async () => {
            const expression = "true && false";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression true && 1 in [2]', async () => {
            const expression = "true && 1 in [2]";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression true && 3 < 3', async () => {
            const expression = "true && 3 < 3";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression true && 3 <= 3 && "bana" != "bana"', async () => {
            const expression = 'true && 3 <= 3 && "bana" != "bana"';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('OR.true statements', async () => {
        it('Should return true when expression true || true', async () => {
            const expression = "true || true";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression true && true || false', async () => {
            const expression = "true && true || false";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression 5 in [1, 4] || 2 > 1', async () => {
            const expression = "5 in [1, 4] || 2 > 1";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression false && false && false || true', async () => {
            const expression = "false && false && false || true";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression false || 3 <= 3 && "bana" != "bana"', async () => {
            const expression = 'false || 3 <= 3 || "bana" != "bana"';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('OR.false statements', async () => {
        it('Should return false when expression false || false', async () => {
            const expression = "false || false";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression false || 1 in [2]', async () => {
            const expression = "false || 1 in [2]";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression false || 3 < 3', async () => {
            const expression = "false || 3 < 3";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression false || 3 < 3 && "bana" != "bana"', async () => {
            const expression = 'false || 3 < 3 || "bana" != "bana"';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Conditional.true statements', async () => {
        it('Should return 2 when expression true ? 2 : 1', async () => {
            const expression = 'true ? 2 : 1';
            const expected = {int64_value: 2};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 8 int64_value when expression true ? (2 + 2 * 2) : 1', async () => {
            const expression = 'true ? (2 + 2 * 2) : 1';
            const expected = {int64_value: 6};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 8 double_value when expression true ? (2.0 + 2.0 * 2.0) : 1', async () => {
            const expression = 'true ? (2.0 + 2.0 * 2.0) : 1';
            const expected = {double_value: 6};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return banana when expression 4 > 3 ? "banana" : 1', async () => {
            const expression = '4 > 3 ? "banana" : 1';
            const expected = {string_value: "banana"};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Conditional.false statements', async () => {
        it('Should return 1 when expression false ? 2 : 1', async () => {
            const expression = 'false ? 2 : 1';
            const expected = {int64_value: 1};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 1 when expression true ? (2 + 2 * 2) : 1', async () => {
            const expression = 'false ? (2 + 2 * 2) : 1';
            const expected = {int64_value: 1};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return banana when expression 4 < 3 ? "banana" : 1', async () => {
            const expression = '4 < 3 ? "banana" : 1';
            const expected = {int64_value: 1};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('Conditional. statements that contain conditional', async () => {
        it('Should return 1 when expression 5 > 6 ? 2 : 4 == 4 ? 1 : 0', async () => {
            const expression = '5 > 6 ? 2 : 4 == 4 ? 1 : 0';
            const expected = {int64_value: 1};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 2 when expression 5 < 6 ? 2 : 4 == 4 ? 1 : 0', async () => {
            const expression = '5 < 6 ? 2 : 4 == 4 ? 1 : 0';
            const expected = {int64_value: 2};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 0 when expression 5 > 6 ? 2 : 4 != 4 ? 1 : 0', async () => {
            const expression = '5 > 6 ? 2 : 4 != 4 ? 1 : 0';
            const expected = {int64_value: 0};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 3 when expression 5 > 6 ? 2 : 4 != 4 ? 1 : 2 > 1 ? 3 : 5', async () => {
            const expression = '5 > 6 ? 2 : 4 != 4 ? 1 : 2 > 1 ? 3 : 5';
            const expected = {int64_value: 3};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('NOT.true statements', async () => {
        it('Should return true when expression !false', async () => {
            const expression = '!false';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression !(false)', async () => {
            const expression = '!(false)';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression !(4 < 3)', async () => {
            const expression = '!(4 < 3)';
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('NOT.false statements', async () => {
        it('Should return false when expression !true', async () => {
            const expression = '!true';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression !(true)', async () => {
            const expression = '!(true)';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression !(4 > 3)', async () => {
            const expression = '!(4 > 3)';
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });
});
