import {beforeEach, describe, expect, it} from "vitest";
import {CelParser} from "../cel_parser";

describe('Macros', async () => {
    let parser: any;

    const values = `{
        "data_points": {
            "banana": {"score": "21.5" },
            "map": {"bla": 1},
            "list": ["test"]
        }
    }`;

    beforeEach(async () => {
        parser = new CelParser();
    })

    describe('has', async () => {
        it('Should return true when expression is has({"bla": 1, "testing": "lala"}.bla)', async () => {
            const expression = "has({\"bla\": 1, \"testing\": \"lala\"}.bla)";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression is has(["bla", 1].bla)', async () => {
            const expression = "has([\"bla\", 1].bla)";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression is has(data_points.map, bla)', async () => {
            const expression = "has(data_points.map, bla)";
            const expected = {bool_value: true};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return true when expression is has(data_points.list, test)', async () => {
            const expression = "has(data_points.list, test)";
            const expected = {bool_value: true};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return true when expression is has(["bla", 1].1)', async () => {
            const expression = "has([\"bla\", 1].1)";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression is has({"bla": 1, "testing": "lala"}, bla)', async () => {
            const expression = "has({\"bla\": 1, \"testing\": \"lala\"}, bla)";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return true when expression is has(["bla", 1], bla)', async () => {
            const expression = "has([\"bla\", 1], bla)";
            const expected = {bool_value: true};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression is has({"bla": 1, "testing": "lala"}.blo)', async () => {
            const expression = "has({\"bla\": 1, \"testing\": \"lala\"}.blo)";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression is has(["bla", 2].1)', async () => {
            const expression = "has([\"bla\", 2].1)";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return false when expression is has(["bla", 2].blo)', async () => {
            const expression = "has([\"bla\", 2].blo)";
            const expected = {bool_value: false};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });

    describe('size', async () => {
        it('Should return 0 when expression is size({})', async () => {
            const expression = "size({})";
            const expected = {int64_value: 0};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 0 when expression is size([])', async () => {
            const expression = "size([])";
            const expected = {int64_value: 0};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 0 when expression is size([]) has spaces around', async () => {
            const expression = "  size([])  ";
            const expected = {int64_value: 0};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 3 when expression is size({"bla": 1, "testing": "lala", "dra": 1})', async () => {
            const expression = "size({\"bla\": 1, \"testing\": \"lala\", \"dra\": 1})";
            const expected = {int64_value: 3};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 4 when expression is size([1, 2, 3, 4])', async () => {
            const expression = "size([1, 2, 3, 4])";
            const expected = {int64_value: 4};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 4 when expression is size(data_points.banana.score) for custom type string', async () => {
            const expression = "size(data_points.banana.score)";
            const expected = {int64_value: 4};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return 1 when expression is size(data_points.map) for custom type map', async () => {
            const expression = "size(data_points.map)";
            const expected = {int64_value: 1};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return 1 when expression is size(data_points.list) for custom type list', async () => {
            const expression = "size(data_points.list)";
            const expected = {int64_value: 1};
            expect(parser.parseStringWithOptions(expression, values)).toStrictEqual(expected);
        });

        it('Should return 4 when expression is size("1234")', async () => {
            const expression = "size(\"1234\")";
            const expected = {int64_value: 4};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 4 when expression is size(\'1234\')', async () => {
            const expression = "size('1234')";
            const expected = {int64_value: 4};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 4 when expression is size(r"1234")', async () => {
            const expression = "size(r\"1234\")";
            const expected = {int64_value: 4};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });

        it('Should return 4 when expression is size(r\'1234\')', async () => {
            const expression = "size(r'1234')";
            const expected = {int64_value: 4};
            expect(parser.parseString(expression)).toStrictEqual(expected);
        });
    });
});
